@@ -2,6 +2,7 @@
 
 import abc
 import datetime
+import json
 import threading
 import time
 import typing
@@ -32,7 +33,7 @@ class Acquisition(abc.ABC):
         self,
         cracker: StatefulCracker,
         trace_count: int = 1000,
-        sample_length: int = 1024,
+        sample_length: int = -1,
         sample_offset: int = 0,
         data_length: int = 0,
         trigger_judge_wait_time: float = 0.05,
@@ -42,6 +43,10 @@ class Acquisition(abc.ABC):
         file_format: str = "scarr",
         file_path: str = "auto",
     ):
+        """
+        :param sample_length: the sample length to be acquired,
+                              it will set to `cracker.current_count().sample_len` if it is -1
+        """
         self._logger = logger.get_logger(self)
         self._last_wave: dict[int, np.ndarray] | None = {1: np.zeros(1)}
         self._status: int = self.STATUS_STOPPED
@@ -285,11 +290,7 @@ class Acquisition(abc.ABC):
         if file_path is not None:
             self.file_path = file_path
         self.pre_init()
-        try:
-            self.init()
-        except Exception as e:
-            self._logger.error("Initialization error: %s", e.args)
-            return
+        self.init()
         self._post_init()
         self._loop(not test, self.file_path, self.file_format)
         self._pre_finish()
@@ -307,6 +308,12 @@ class Acquisition(abc.ABC):
 
         cracker_version = self.cracker.get_version()
         if persistent:
+            channel_count = len(self.cracker.get_current_config().osc_analog_channel_enable.keys())
+            if self.sample_length == -1:
+                sample_length = self.cracker.get_current_config().osc_sample_len
+            else:
+                sample_length = self.sample_length
+
             if file_path is None or file_path == "auto":
                 file_path = "./dataset/" + datetime.datetime.now().strftime("%Y%m%d%H%M%S")
                 if file_format == "scarr":
@@ -317,23 +324,24 @@ class Acquisition(abc.ABC):
             if file_format == "scarr":
                 dataset = ScarrTraceDataset.new(
                     file_path,
-                    1,
+                    channel_count,
                     self.trace_count,
-                    self.sample_length,
+                    sample_length,
                     self.data_length,
                     version=f"({cracker_version}, {cracker_version})",
                 )  # todo channel
             elif file_format == "numpy":
                 dataset = NumpyTraceDataset.new(
                     file_path,
-                    1,
+                    channel_count,
                     self.trace_count,
-                    self.sample_length,
+                    sample_length,
                     self.data_length,
                     version=f"({cracker_version}, {cracker_version})",
                 )  # todo channel
             else:
-                self._logger.warning(f"Unsupported file format: {file_format}")
+                self._logger.error(f"Unsupported file format: {file_format}")
+                return
         else:
             dataset = None
 
@@ -352,6 +360,7 @@ class Acquisition(abc.ABC):
                 do_error_count += 1
                 if self.do_error_handler_strategy == self.DO_ERROR_HANDLER_STRATEGY_EXIT:
                     self._logger.error("Exit with do error")
+                    break
                 elif do_error_count > self.do_error_max_count:
                     self._logger.error(
                         f"Exit with do function get error count: {do_error_count} is grate than do_error_max_count"
@@ -372,8 +381,7 @@ class Acquisition(abc.ABC):
                     )
                     break
                 self._logger.debug("Judge trigger status.")
-                # if self._is_triggered():
-                if True:
+                if self._is_triggered():
                     self._last_wave = self._get_waves(self.sample_offset, self.sample_length)
                     if self._last_wave is not None:
                         self._logger.debug(
@@ -390,8 +398,10 @@ class Acquisition(abc.ABC):
 
                 time.sleep(self.trigger_judge_wait_time)
             if dataset is not None and self._last_wave is not None:
-                dataset.set_trace(0, trace_index, self._last_wave[1], np.frombuffer(data, dtype=np.uint8))
-                # dataset.set_trace(1, trace_index, self._last_wave[2], data)  # todo second channel
+                if 1 in self._last_wave.keys():
+                    dataset.set_trace(0, trace_index, self._last_wave[1], np.frombuffer(data, dtype=np.uint8))
+                elif 2 in self._last_wave.keys():
+                    dataset.set_trace(1, trace_index, self._last_wave[2], np.frombuffer(data, dtype=np.uint8))
             self._post_do()
             trace_index += 1
             self._current_trace_count = trace_index
@@ -486,26 +496,22 @@ class Acquisition(abc.ABC):
     def _post_do(self): ...
 
     def _is_triggered(self):
-        return self.cracker.osc_is_triggered()
+        status, res = self.cracker.osc_is_triggered()
+        res_code = int.from_bytes(res, "big")
+        return res_code == 4
 
-    def _get_waves(self, offset: int, sample_count: int) -> dict[int, np.ndarray]:
+    def _get_waves(self, offset: int, sample_length: int) -> dict[int, np.ndarray]:
+        if sample_length == -1:
+            sample_length = self.cracker.get_current_config().osc_sample_len
         config = self.cracker.get_current_config()
         if config.osc_analog_channel_enable is None:
             raise Exception("Channel info can't be none.")
         enable_channels = [k for k, v in config.osc_analog_channel_enable.items() if v]
         wave_dict = {}
         for c in enable_channels:
-            status, wave_dict[c] = self.cracker.osc_get_analog_wave(c, offset, sample_count)
+            status, wave_dict[c] = self.cracker.osc_get_analog_wave(c, offset, sample_length)
         return wave_dict
 
-    def save_data(self, data_type: str = "zarr"):
-        """
-        Save wave, key and other info to acquisition dataset file.
-        """
-        # if data_type == 'zarr':
-        #     zarr.create()
-        ...
-
     def _pre_finish(self): ...
 
     def _post_finish(self): ...
@@ -518,6 +524,49 @@ class Acquisition(abc.ABC):
     def get_last_wave(self) -> dict[int, np.ndarray] | None:
         return self._last_wave
 
+    def dump_config(self, path: str = None) -> str | None:
+        """
+        Dump the current config to a JSON file if a path is specified, or to a JSON string if no path is specified.
+        :param path: the path to the JSON file
+        :return: the content of JSON string or None if no path is specified.
+        """
+        config = {
+            "trace_count": self.trace_count,
+            "sample_length": self.sample_length,
+            "sample_offset": self.sample_offset,
+            "trigger_judge_timeout": self.trigger_judge_timeout,
+            "trigger_judge_wait_time": self.trigger_judge_wait_time,
+            "do_error_max_count": self.do_error_max_count,
+            "trace_file_path": self.file_path,
+        }
+        if path is None:
+            return json.dumps(config)
+        else:
+            with open(path, "w") as f:
+                json.dump(config, f)
+
+    def load_config_from_file(self, path: str) -> None:
+        """
+        Load config from a JSON file
+        :param path: the path to the JSON file
+        """
+        with open(path) as f:
+            self.load_config_from_str(json.load(f))
+
+    def load_config_from_str(self, json_str: str) -> None:
+        """
+        Load config from a JSON string
+        :param json_str: the JSON string
+        """
+        config = json.loads(json_str)
+        self.trace_count = config["trace_count"]
+        self.sample_length = config["sample_length"]
+        self.sample_offset = config["sample_offset"]
+        self.trigger_judge_timeout = config["trigger_judge_timeout"]
+        self.trigger_judge_wait_time = config["trigger_judge_wait_time"]
+        self.do_error_max_count = config["do_error_max_count"]
+        self.file_path = config["trace_file_path"]
+
     @staticmethod
     def builder():
         return AcquisitionBuilder()
