@@ -1,21 +1,29 @@
 # Copyright 2024 CrackNuts. All rights reserved.
 
 import abc
+import json
+import logging
+import os
+import re
 import socket
 import struct
 import threading
 import typing
 from abc import ABC
-from dataclasses import dataclass
 
 import numpy as np
+from packaging.version import Version
 
+import cracknuts
 import cracknuts.utils.hex_util as hex_util
 from cracknuts import logger
 from cracknuts.cracker import protocol
+from cracknuts.cracker.operator import Operator
 
 
 class Cracker(typing.Protocol):
+    """Interface of Cracker classes."""
+
     def get_default_config(self) -> "Config": ...
 
     def set_addr(self, ip, port) -> None: ...
@@ -24,7 +32,10 @@ class Cracker(typing.Protocol):
 
     def get_uri(self): ...
 
-    def connect(self): ...
+    def connect(
+        self,
+        update_bin: bool = True,
+    ): ...
 
     def disconnect(self): ...
 
@@ -36,7 +47,17 @@ class Cracker(typing.Protocol):
 
     def send_with_command(
         self, command: int, rfu: int = 0, payload: str | bytes | None = None
-    ) -> tuple[int, bytes | None]: ...
+    ) -> tuple[int, bytes | None]:
+        """Send payload with command
+
+        :param command:  the command to send.
+        :type command: int
+        :param rfu:       the RFU to send.
+        :type rfu: int
+        :param payload: the payload to send.
+        :type payload: bytes | None
+        """
+        ...
 
     def echo(self, payload: str) -> str: ...
 
@@ -46,10 +67,13 @@ class Cracker(typing.Protocol):
 
     def get_name(self) -> str | None: ...
 
-    def get_version(self) -> str | None: ...
+    def get_version(self) -> str | None:
+        """Get cracker version"""
+        ...
 
     def cracker_read_register(self, base_address: int, offset: int) -> tuple[int, bytes | None]:
         """Read register
+
         :param base_address: the base address of the register
         :param offset: the offset of the register
         :return: the result of reading the register
@@ -60,6 +84,7 @@ class Cracker(typing.Protocol):
         self, base_address: int, offset: int, data: bytes | int | str
     ) -> tuple[int, bytes | None]:
         """Write register
+
         :param base_address: the base address of the register
         :param offset: the offset of the register
         :param data: the data to be written
@@ -79,12 +104,16 @@ class Cracker(typing.Protocol):
 
     def osc_set_digital_voltage(self, voltage: int): ...
 
-    def osc_set_trigger_mode(self, source: int, stop: int): ...
+    def osc_set_trigger_mode(self, mode: int): ...
 
-    def osc_set_analog_trigger_source(self, channel: int): ...
+    def osc_set_analog_trigger_source(self, source: int): ...
 
     def osc_set_digital_trigger_source(self, channel: int): ...
 
+    def osc_set_trigger_edge(self, edge: int | str): ...
+
+    def osc_set_trigger_edge_level(self, edge: int): ...
+
     def osc_set_analog_trigger_voltage(self, voltage: int): ...
 
     def osc_set_sample_delay(self, delay: int): ...
@@ -199,11 +228,14 @@ class Commands:
     OSC_DIGITAL_CHANNEL_ENABLE = 0x0110
     OSC_DIGITAL_VOLTAGE = 0x0111
 
-    OSC_TRIGGER_MODE = 0x0120
+    OSC_TRIGGER_MODE = 0x0151
 
-    OSC_ANALOG_TRIGGER_SOURCE = 0x0121
+    OSC_ANALOG_TRIGGER_SOURCE = 0x0150
     OSC_DIGITAL_TRIGGER_SOURCE = 0x0122
 
+    OSC_TRIGGER_EDGE = 0x152
+    OSC_TRIGGER_EDGE_LEVEL = 0x153
+
     OSC_ANALOG_TRIGGER_VOLTAGE = 0x0123
 
     OSC_SAMPLE_DELAY = 0x0124
@@ -248,7 +280,6 @@ class Commands:
     CRACKER_CA_DATA = 0x025A
 
 
-@dataclass
 class Config:
     def __init__(
         self,
@@ -285,13 +316,30 @@ class Config:
         """
         self._binder[key] = callback
 
+    def __str__(self):
+        return f"Config({", ".join([f"{k}: {v}" for k, v in self.__dict__.items()])})"
+
+    def dump_to_json(self) -> str:
+        return json.dumps({k: v for k, v in self.__dict__.items() if k != "_binder"})
+
+    def load_from_json(self, json_str: str) -> "Config":
+        for k, v in json.loads(json_str).items():
+            if k == "osc_analog_channel_enable":
+                v = {int(_k): _v for _k, _v in v.items()}
+            self.__dict__[k] = v
+        return self
+
 
 class AbsCnpCracker(ABC, Cracker):
-    """Cracker
-    Cracker
-    """
+    """Abstract cnp protocol supported Cracker"""
 
-    def __init__(self, address: tuple | str | None = None):
+    def __init__(
+        self,
+        address: tuple | str | None = None,
+        bin_server_path: str | None = None,
+        bin_bitstream_path: str | None = None,
+        operator_port: int = None,
+    ):
         """
         :param address: Cracker device address (ip, port) or "cnp://xxx:xx"
         """
@@ -307,6 +355,9 @@ class AbsCnpCracker(ABC, Cracker):
             0: False,
             1: False,
         }
+        self._bin_server_path = bin_server_path
+        self._bin_bitstream_path = bin_bitstream_path
+        self._operator_port = operator_port
 
     @abc.abstractmethod
     def get_default_config(self) -> Config: ...
@@ -332,11 +383,14 @@ class AbsCnpCracker(ABC, Cracker):
         else:
             return f"cnp://{self._server_address[0]}:{self._server_address[1]}"
 
-    def connect(self):
+    def connect(self, update_bin: bool = True):
         """
         Connect to Cracker device.
         :return: Cracker self.
         """
+        if update_bin:
+            self._update_cracker_bin(self._bin_server_path, self._bin_bitstream_path, self._operator_port)
+
         try:
             if not self._socket:
                 self._socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
@@ -351,6 +405,95 @@ class AbsCnpCracker(ABC, Cracker):
             self._logger.error("Connection failed: %s", e)
             self._connection_status = False
 
+    def _update_cracker_bin(
+        self,
+        bin_server_path: str | None = None,
+        bin_bitstream_path: str | None = None,
+        operator_port: int = None,
+        server_version: str = None,
+        bitstream_version: str = None,
+    ):
+        if operator_port is None:
+            operator_port = protocol.DEFAULT_OPERATOR_PORT
+        operator = Operator(self._server_address[0], operator_port)
+        operator.connect()
+
+        if operator.get_status():
+            operator.disconnect()
+            return
+
+        hardware_model = operator.get_hardware_model()
+
+        bin_path = os.path.join(cracknuts.__file__, "bin")
+        user_home_bin_path = os.path.join(os.path.expanduser("~"), ".cracknuts", ".bin")
+        current_bin_path = os.path.join(os.getcwd(), ".bin")
+
+        if bin_server_path is None or bin_bitstream_path is None:
+            server_bin_dict, bitstream_bin_dict = self._find_bin_files(bin_path, user_home_bin_path, current_bin_path)
+            if bin_server_path is None:
+                bin_server_path = self._get_version_file_path(server_bin_dict, hardware_model, server_version)
+            if bin_bitstream_path is None:
+                bin_bitstream_path = self._get_version_file_path(bitstream_bin_dict, hardware_model, bitstream_version)
+
+        if not os.path.exists(bin_server_path):
+            raise FileNotFoundError(f"Server binary file not found: {bin_server_path}")
+
+        if not os.path.exists(bin_bitstream_path):
+            raise FileNotFoundError(f"Bitstream file not found: {bin_bitstream_path}")
+
+        bin_server = open(bin_server_path, "rb").read()
+        bin_bitstream = open(bin_bitstream_path, "rb").read()
+
+        try:
+            if not operator.update_server(bin_server):
+                raise Exception("Failed to update cracker server")
+            if not operator.update_bitstream(bin_bitstream):
+                raise Exception("Failed to update cracker bitstream")
+            if not operator.start_server():
+                raise Exception("Failed to start cracker server")
+            if not operator.get_status():
+                raise Exception("Failed to update and start cracker")
+        finally:
+            operator.disconnect()
+
+    @staticmethod
+    def _get_version_file_path(bin_dict: dict[str, dict[str, str]], hardware_model: str, version: str) -> str:
+        dict_by_hardware = bin_dict.get(hardware_model, None)
+        if dict_by_hardware is None:
+            raise Exception(f"can't find bin file for hardware model: {hardware_model}.")
+        if version is None:
+            sorted_version = sorted(dict_by_hardware.keys(), key=Version)
+            version = sorted_version[-1]
+        return dict_by_hardware.get(version, None)
+
+    @staticmethod
+    def _find_bin_files(*bin_paths: str) -> tuple[dict[str, dict[str, str]], dict[str, dict[str, str]]]:
+        server_path_pattern = r"server-(?P<hardware>.+?)-(?P<firmware>.+?)"
+        bitstream_path_pattern = r"bitstream-(?P<hardware>.+?)-(?P<firmware>.+?).bit.bin"
+
+        server_bin_dict = {}
+        bitstream_bin_dict = {}
+
+        for bin_path in bin_paths:
+            if os.path.exists(bin_path):
+                for filename in os.listdir(bin_path):
+                    server_match = re.search(server_path_pattern, filename)
+                    if server_match:
+                        server_hardware_version = server_match.group("hardware")
+                        server_firmware_version = server_match.group("firmware")
+                        server_hardware_dict = server_bin_dict.get(server_hardware_version, {})
+                        server_hardware_dict[server_firmware_version] = os.path.join(bin_path, filename)
+                        server_bin_dict[server_hardware_version] = server_hardware_dict
+                    bitstream_match = re.search(bitstream_path_pattern, filename)
+                    if bitstream_match:
+                        bitstream_hardware_version = bitstream_match.group("hardware")
+                        bitstream_firmware_version = bitstream_match.group("firmware")
+                        bitstream_hardware_dict = bitstream_bin_dict.get(bitstream_hardware_version, {})
+                        bitstream_hardware_dict[bitstream_firmware_version] = os.path.join(bin_path, filename)
+                        bitstream_bin_dict[bitstream_hardware_version] = bitstream_hardware_dict
+
+        return server_bin_dict, bitstream_bin_dict
+
     def disconnect(self):
         """
         Disconnect Cracker device.
@@ -395,18 +538,22 @@ class AbsCnpCracker(ABC, Cracker):
             if not self.get_connection_status():
                 self._logger.error("Cracker is not connected.")
                 return protocol.STATUS_ERROR, None
-            self._logger.debug(f"Send message to {self._server_address}: \n{hex_util.get_bytes_matrix(message)}")
+            if self._logger.isEnabledFor(logging.DEBUG):
+                self._logger.debug(f"Send message to {self._server_address}: \n{hex_util.get_bytes_matrix(message)}")
             self._socket.sendall(message)
             resp_header = self._socket.recv(protocol.RES_HEADER_SIZE)
-            self._logger.debug(
-                "Get response header from %s: \n%s",
-                self._server_address,
-                hex_util.get_bytes_matrix(resp_header),
-            )
+            if self._logger.isEnabledFor(logging.DEBUG):
+                self._logger.debug(
+                    "Get response header from %s: \n%s",
+                    self._server_address,
+                    hex_util.get_bytes_matrix(resp_header),
+                )
             magic, version, direction, status, length = struct.unpack(protocol.RES_HEADER_FORMAT, resp_header)
-            self._logger.debug(
-                f"Receive header from {self._server_address}: {magic}, {version}, {direction}, {status:02X}, {length}"
-            )
+            if self._logger.isEnabledFor(logging.DEBUG):
+                self._logger.debug(
+                    f"Receive header from {self._server_address}: "
+                    f"{magic}, {version}, {direction}, {status:02X}, {length}"
+                )
             if status >= protocol.STATUS_ERROR:
                 self._logger.error(f"Receive status error: {status:02X}")
             if length == 0:
@@ -417,9 +564,10 @@ class AbsCnpCracker(ABC, Cracker):
                     f"Receive payload from {self._server_address}: \n{hex_util.get_bytes_matrix(resp_payload)}"
                 )
             else:
-                self._logger.debug(
-                    f"Receive payload from {self._server_address}: \n{hex_util.get_bytes_matrix(resp_payload)}"
-                )
+                if self._logger.isEnabledFor(logging.DEBUG):
+                    self._logger.debug(
+                        f"Receive payload from {self._server_address}: \n{hex_util.get_bytes_matrix(resp_payload)}"
+                    )
             return status, resp_payload
         except OSError as e:
             self._logger.error("Send message failed: %s, and msg: %s", e, message)
@@ -449,7 +597,8 @@ class AbsCnpCracker(ABC, Cracker):
         if self._socket is not None:
             self._socket.sendall(payload.encode("ascii"))
             res = self._socket.recv(1024).decode("ascii")
-            self._logger.debug(f"Get response: {res}")
+            if self._logger.isEnabledFor(logging.DEBUG):
+                self._logger.debug(f"Get response: {res}")
             return res
         else:
             return "Cracker not connected."
@@ -462,7 +611,8 @@ class AbsCnpCracker(ABC, Cracker):
             content = bytes.fromhex(payload)
             self._socket.sendall(content)
             res = self._socket.recv(1024).hex()
-            self._logger.debug(f"Get response: {res}")
+            if self._logger.isEnabledFor(logging.DEBUG):
+                self._logger.debug(f"Get response: {res}")
             return res
         else:
             return "Cracker not connected."
@@ -503,14 +653,20 @@ class AbsCnpCracker(ABC, Cracker):
     def osc_set_digital_voltage(self, voltage: int): ...
 
     @abc.abstractmethod
-    def osc_set_trigger_mode(self, source: int, stop: int): ...
+    def osc_set_trigger_mode(self, mode: int): ...
 
     @abc.abstractmethod
-    def osc_set_analog_trigger_source(self, channel: int): ...
+    def osc_set_analog_trigger_source(self, source: int): ...
 
     @abc.abstractmethod
     def osc_set_digital_trigger_source(self, channel: int): ...
 
+    @abc.abstractmethod
+    def osc_set_trigger_edge(self, edge: int | str): ...
+
+    @abc.abstractmethod
+    def osc_set_trigger_edge_level(self, edge: int): ...
+
     @abc.abstractmethod
     def osc_set_analog_trigger_voltage(self, voltage: int): ...
 
