@@ -7,13 +7,18 @@ import time
 import typing
 
 import numpy as np
-import zarr # type: ignore
+import zarr  # type: ignore
 
 from cracknuts import logger
 
 
 class TraceDatasetData:
-    def __init__(self, get_trace_data: typing.Callable[[typing.Any, typing.Any], tuple], level: int = 0, index: tuple | None = None):
+    def __init__(
+        self,
+        get_trace_data: typing.Callable[[typing.Any, typing.Any], tuple],
+        level: int = 0,
+        index: tuple | None = None,
+    ):
         self._level: int = level
         self._index: tuple | None = index
         self._get_trace_data = get_trace_data
@@ -126,6 +131,7 @@ class ScarrTraceDataset(TraceDataset):
         trace_count: int | None = None,
         sample_count: int | None = None,
         data_length: int | None = None,
+        trace_dtype: np.dtype = np.int16,
         zarr_kwargs: dict | None = None,
         zarr_trace_group_kwargs: dict | None = None,
         zarr_data_group_kwargs: dict | None = None,
@@ -151,7 +157,12 @@ class ScarrTraceDataset(TraceDataset):
         self._zarr_data = zarr.open(zarr_path, mode=mode, **zarr_kwargs)
 
         if create_empty:
-            if self._channel_count is None or self._trace_count is None or self._sample_count is None or self._data_length is None:
+            if (
+                self._channel_count is None
+                or self._trace_count is None
+                or self._sample_count is None
+                or self._data_length is None
+            ):
                 raise ValueError(
                     "channel_count and trace_count and sample_count and data_length "
                     "must be specified when in write mode."
@@ -163,7 +174,7 @@ class ScarrTraceDataset(TraceDataset):
                 channel_group.create(
                     self._ARRAY_TRACES_PATH,
                     shape=(self._trace_count, self._sample_count),
-                    dtype=np.int16,
+                    dtype=trace_dtype,
                     **zarr_trace_group_kwargs,
                 )
                 channel_group.create(
@@ -231,7 +242,8 @@ class ScarrTraceDataset(TraceDataset):
         if trace_index not in range(0, self._trace_count):
             raise ValueError("trace, index out of range")
         self._get_under_root(channel_index, self._ARRAY_TRACES_PATH)[trace_index] = trace
-        self._get_under_root(channel_index, self._ARRAY_DATA_PATH)[trace_index] = data
+        if self._data_length != 0 and data is not None:
+            self._get_under_root(channel_index, self._ARRAY_DATA_PATH)[trace_index] = data
 
     def get_origin_data(self):
         return self._zarr_data
@@ -284,6 +296,7 @@ class NumpyTraceDataset(TraceDataset):
         channel_count: int | None = None,
         trace_count: int | None = None,
         sample_count: int | None = None,
+        trace_dtype: np.dtype = np.int16,
         data_length: int | None = None,
         create_time: int | None = None,
         version: str | None = None,
@@ -305,13 +318,18 @@ class NumpyTraceDataset(TraceDataset):
         self._data_array: np.ndarray
 
         if create_empty:
-            if self._channel_count is None or self._trace_count is None or self._sample_count is None or self._data_length is None:
+            if (
+                self._channel_count is None
+                or self._trace_count is None
+                or self._sample_count is None
+                or self._data_length is None
+            ):
                 raise ValueError(
                     "channel_count and trace_count and sample_count and data_length "
                     "must be specified when in write mode."
                 )
             self._trace_array = np.zeros(
-                shape=(self._channel_count, self._trace_count, self._sample_count), dtype=np.int16
+                shape=(self._channel_count, self._trace_count, self._sample_count), dtype=trace_dtype
             )
             self._data_array = np.zeros(
                 shape=(self._channel_count, self._trace_count, self._data_length), dtype=np.uint8
@@ -404,6 +422,7 @@ class NumpyTraceDataset(TraceDataset):
             trace_count=trace_count,
             sample_count=sample_count,
             data_length=data_length,
+            trace_dtype=trace.dtype,
         )
 
         if array_size == 1:
@@ -461,7 +480,7 @@ class NumpyTraceDataset(TraceDataset):
 
     def set_trace(self, channel_index: int, trace_index: int, trace: np.ndarray, data: np.ndarray | None):
         self._trace_array[channel_index, trace_index, :] = trace
-        if data is not None:
+        if self._data_length != 0 and data is not None:
             self._data_array[channel_index, trace_index, :] = data
 
     def _get_trace_data(self, channel_slice, trace_slice) -> tuple[list, list, np.ndarray, np.ndarray]:
